package sc;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Esta clase sirve de clase de control para el interfaz gráfico de una calculadora sencilla.
 * Sólo ejecuta las cuatro operaciones aritméticas básicas.
 * Trabaja con aritmética decimal en base 10.
 * Permite borrar digitos introducidos y borrar todo (resetear)
 * 
 * @author German
 * @version 1.0 (oct 2012)
 */

public class CalculadoraElectronica{
    private CalcuGUI gui;
    
    public CalculadoraElectronica(CalcuGUI gui) {
        this.gui=gui;
        gui.subscribirABotones(new ActionListener(){
            @Override
            public void actionPerformed(ActionEvent evt) {
                char c;
                switch (c=((javax.swing.JButton)evt.getSource()).getText().charAt(0) ) {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9': digito(c); break;
                    case '.': coma();break;
                    case '+': 
                    case '-':
                    case 'x':
                    case '/': 
                    case '=': opera(c); break;
                    case 'C': reset(); break;  
                    case '<': atras(); break;  
                }
            }
        });  
    }
    
    
    
    //<editor-fold defaultstate="collapsed" desc="ESTADO">
    /** La aritmética la hace una ALU (en realidad sin "L" porque no hay operaciones lógicas */
    private ALU alu=new ALU();
    
    /** La introducción de un digito puede estar en clear, en la parte entera o en la parte decimal */
    private enum EstadoEscritura {RESET,ENTERO, DECIMAL}
    
    /** Estado de introducción de un número */
    private EstadoEscritura estadoEscritura=EstadoEscritura.RESET;
    
    /** Operador pendiente
     *  Si es el "=" no hay nada pendiente. Si es otro, se está a la espera de un número para aplicarlo con el acumulador de la ALU
     */
    private char ultimoOperador='=';
    
    /** String de construcción de un número */
    private String numeroEnPantalla="0.0";
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="COMPORTAMIENTO">
    /** Los digitos se encadenan, controlando que si el estado es clear, se borra la representación actual y se entra en estado "ENTERO"
     * 
     * @param c el carácter que representa al dígito pulsado
     */
    void digito(char c){
        if (estadoEscritura==EstadoEscritura.RESET) {
            numeroEnPantalla="";
            estadoEscritura=EstadoEscritura.ENTERO;
        }
        gui.setDisplay(numeroEnPantalla+=c);
    }
    
    /** La coma pasa al estado DECIMAL. En RESET inicializa número a "0." y en ENTERO añade ".". En DECIMAL no hace nada.
     */
    void coma() {
        switch(estadoEscritura) {
            case RESET: numeroEnPantalla="0.";break;
            case ENTERO: numeroEnPantalla+=".";break;
        }
        estadoEscritura=EstadoEscritura.DECIMAL;
        gui.setDisplay(numeroEnPantalla);
    }
    
    /** Reseteo de la calculadora.
     * 
     */
    void reset() {
        ultimoOperador='=';
        alu.clear();
        estadoEscritura=EstadoEscritura.RESET;
        gui.setDisplay(numeroEnPantalla="0.0");
        
    }
    
    /** Borrado del último caracter representado en pantalla.
     */
    void atras() {
        if (estadoEscritura==EstadoEscritura.RESET) {
            gui.setDisplay(numeroEnPantalla);
            return;
        }
        if (estadoEscritura==EstadoEscritura.DECIMAL && numeroEnPantalla.endsWith(".")) estadoEscritura=EstadoEscritura.ENTERO;
        if (numeroEnPantalla.length()==1) {
            estadoEscritura=EstadoEscritura.RESET;
            gui.setDisplay(numeroEnPantalla="0.0");
            return;
        }
        gui.setDisplay(numeroEnPantalla=numeroEnPantalla.substring(0,numeroEnPantalla.length()-1));
    }
    
    /** Atención a los operadores " = + - x / ".
     * 
     *  <ul>
     *  <li>Da por terminada la introduccion de un número poniendo estado RESET</li>
     *  <li>Lleva al número al registro R de la ALU y aplica la operación pendiente</li>
     *  <li>La nueva operación queda como pendiente</li>
     *  </ul>
     * 
     * @param operador el carácter que representa al operador pulsado
     */
     void opera(char operador) {
        estadoEscritura=EstadoEscritura.RESET;
        alu.setRegistro(Double.parseDouble(numeroEnPantalla));
        switch (ultimoOperador) {
            case '=':alu.clear();alu.suma();break;
            case '+':alu.suma();break;
            case '-':alu.resta();break;
            case 'x':alu.multiplica();break;
            case '/':alu.divide();break;
        }
        ultimoOperador=operador;
        gui.setDisplay(numeroEnPantalla=String.valueOf(alu.getAcumulador()));
    }
    //</editor-fold>
    
}
